from typing import Literal

import numpy as np
import pandas as pd

from .errors_exceptions import InvalidArgumentError
from .moving_average import sma, ema, sema, rma


def RSI(
    source: pd.Series,
    periods: int = 14,
    ma_method: Literal["sma", "ema", "dema", "tema", "rma"] = "rma",
) -> pd.Series:
    """
    Calculate the Relative Strength Index (RSI) for a given time series
    data.

    Parameters:
    -----------
    source : pd.Series
        The input time series data for which to calculate RSI.
    periods : int, optional
        The number of periods to use for RSI calculation.
        (default: 14)

    Returns:
    --------
    pd.Series
        The calculated RSI values for the input data.
    """
    upward_diff = pd.Series(np.maximum(source - source.shift(1), 0.0)).dropna()

    downward_diff = (
        pd.Series(np.maximum(source.shift(1) - source, 0.0)).dropna()
    )

    match ma_method:
        case "sma":
            relative_strength = sma(upward_diff, periods) / sma(downward_diff, periods)
        case "ema":
            relative_strength = ema(upward_diff, periods) / ema(downward_diff, periods)
        case "dema":
            relative_strength = sema(upward_diff, periods, 2) / sema(downward_diff, periods, 2)
        case "tema":
            relative_strength = sema(upward_diff, periods, 3) / sema(downward_diff, periods, 3)
        case "rma":
            relative_strength = rma(upward_diff, periods) / rma(downward_diff, periods)
        case _:
            raise InvalidArgumentError(
                "ma_method must be 'sma', 'ema', 'dema', 'tema', or 'rma',"
                f" got '{ma_method}'."
            )

    rsi = 100 - (100 / (1 + relative_strength))
    return rsi.rename("RSI")



from typing import Literal
import pandas as pd
import numpy as np

from .errors_exceptions import InvalidArgumentError
from .moving_average import sma, ema, sema, rma
from .utils import DynamicTimeWarping


def bollinger_bands(
    source: pd.Series,
    length: int,
    mult: float,
    ma_method: Literal["sma", "ema", "dema", "tema", "rma"] = "ema",
) -> pd.DataFrame:
    match ma_method:
        case "sma":
            basis = sma(source, length)
        case "ema":
            basis = ema(source, length)
        case "dema":
            basis = sema(source, length, 2)
        case "tema":
            basis = sema(source, length, 3)
        case "rma":
            basis = rma(source, length)
        case _:
            raise InvalidArgumentError(
                "ma_method must be 'sma', 'ema', 'dema', 'tema', or 'rma',"
                f" got '{ma_method}'."
            )

    deviation = mult * source.rolling(window=length).std()

    return pd.DataFrame(
        {
            "basis": basis,
            "upper": basis + deviation,
            "lower": basis - deviation,
        }
    )


def bollinger_trends(
    source: pd.Series,
    short_length: int = 20,
    long_length: int = 50,
    mult: float = 2,
    ma_method: Literal["sma", "ema", "dema", "tema", "rma"] = "sma",
    stdev_method: Literal["absolute", "ratio", "dtw"] = "absolute",
    diff_method: Literal["normal", "absolute", "ratio", "dtw"] = "normal",
    based_on: Literal["short_length", "long_length"] = "short_length",
) -> pd.Series:
    short_bands = bollinger_bands(source, short_length, mult, ma_method)
    long_bands = bollinger_bands(source, long_length, mult, ma_method)

    short_lower = short_bands["lower"]
    short_upper = short_bands["upper"]

    long_lower = long_bands["lower"]
    long_upper = long_bands["upper"]

    match based_on:
        case "short_length":
            middle = short_bands["basis"]
        case "long_length":
            middle = long_bands["basis"]
        case _:
            raise InvalidArgumentError(
                "based_on must be 'short_length' or 'long_length'."
                f" got '{based_on}'."
            )

    short_length_index = short_length - 1

    match stdev_method:
        case "absolute":
            lower_diff = abs(short_lower - long_lower)
            upper_diff = abs(short_upper - long_upper)

        case "ratio":
            lower_diff = abs(short_lower / long_lower)
            upper_diff = abs(short_upper / long_upper)

        case "dtw":
            short_lower = short_lower.iloc[short_length_index:]
            short_upper = short_upper.iloc[short_length_index:]

            long_lower = long_lower.iloc[short_length_index:]
            long_upper = long_upper.iloc[short_length_index:]

            lower_diff = abs(
                DynamicTimeWarping(short_lower, long_lower)
                .calculate_dtw_distance("ratio", True)
            )

            upper_diff = abs(
                DynamicTimeWarping(short_upper, long_upper)
                .calculate_dtw_distance("ratio", True)
            )
        case _:
            raise InvalidArgumentError(
                "stdev_method must be 'absolute', 'ratio', or 'dtw'."
                f" got '{stdev_method}'."
            )

    match diff_method:
        case "normal":
            return (
                (lower_diff - upper_diff) / middle * 100
            ).rename("Bollinger Trend")
        case "absolute":
            return (
                (lower_diff - upper_diff) - middle * 100
            ).rename("Bollinger Trend")
        case "ratio":
            return (
                (lower_diff / upper_diff) / middle * 100
            ).rename("Bollinger Trend")
        case "dtw":
            middle = middle.iloc[short_length_index:]

            distance_diff = (
                DynamicTimeWarping(lower_diff, upper_diff)
                .calculate_dtw_distance("ratio", True)
            ) * 100

            null_series = pd.Series(
                np.full(short_length_index, np.nan),
                index=range(0, short_length_index),
            )

            return pd.concat(
                [
                    null_series, (
                        DynamicTimeWarping(distance_diff, middle)
                        .calculate_dtw_distance("ratio", True)  * 100
                    )
                ]
            ).rename("Bollinger Trend")

        case _:
            raise InvalidArgumentError(
                "diff_method must be 'normal', 'absolute', 'ratio', or 'dtw'."
                f" got '{diff_method}'."
            )


import unittest
import pandas as pd
import numpy as np
from src.tradingview_indicators.SMIO import SMIO
from src.tradingview_indicators.errors_exceptions import InvalidArgumentError


class TestSMIO(unittest.TestCase):
    def setUp(self):
        np.random.seed(42)
        self.source = pd.Series(np.random.randint(1, 500, 37))
        self.dema_source = pd.Series(np.random.randint(1, 500, 42))
        self.tema_source = pd.Series(np.random.randint(1, 500, 52))
        self.long_length = 20
        self.short_length = 5
        self.signal_length = 5

    def test_smio_sma(self):
        expected_result = pd.Series(
            [
                np.nan,
                np.nan,
                np.nan,
                np.nan,
                -0.03702186972261484,
                -0.005783453565452512,
                -0.016429866077644253,
                -0.04027520292196353,
                0.005410234847397045,
                -0.0022208095652088804,
                -0.04643814522873069,
                -0.025121318783773383,
                -0.018804672815918785,
            ],
            index=range(24, 37),
            name="SMIO",
        )

        result = SMIO(
            self.source,
            long_length=self.long_length,
            short_length=self.short_length,
            signal_length=self.signal_length,
            ma_method="sma",
        )

        pd.testing.assert_series_equal(result, expected_result)

    def test_smio_ema(self):
        expected_result = pd.Series(
            [
                np.nan,
                np.nan,
                np.nan,
                np.nan,
                -0.03702186972261484,
                -0.006788511334759897,
                -0.0185668188832111,
                -0.037422277778225675,
                0.000891031289350537,
                -0.004603580707847523,
                -0.04452781559367561,
                -0.023386574709263256,
                -0.01395680596492259,
            ],
            index=range(24, 37),
            name="SMIO",
        )

        result = SMIO(
            self.source,
            long_length=self.long_length,
            short_length=self.short_length,
            signal_length=self.signal_length,
            ma_method="ema",
        )

        pd.testing.assert_series_equal(result, expected_result)

    def test_smio_dema(self):
        expected_result = pd.Series(
            [
                np.nan,
                np.nan,
                np.nan,
                np.nan,
                np.nan,
                np.nan,
                np.nan,
                np.nan,
                0.0267577196807172,
                -0.005846336095866194,
                0.02265095765153298,
                0.029665062791784838,
                -0.0031684075739063684,
                0.017273292479976096,
                0.03715191104254778,
                0.04659186566896591,
                0.04402397777194027,
                0.012004380888481284,
            ],
            index=range(24, 42),
            name="SMIO",
        )

        result = SMIO(
            self.dema_source,
            long_length=self.long_length,
            short_length=self.short_length,
            signal_length=self.signal_length,
            ma_method="dema",
        )

        pd.testing.assert_series_equal(result, expected_result)

    def test_smio_tema(self):
        expected_result = pd.Series(
            [
                np.nan,
                np.nan,
                np.nan,
                np.nan,
                np.nan,
                np.nan,
                np.nan,
                np.nan,
                np.nan,
                np.nan,
                np.nan,
                np.nan,
                0.011979480399899049,
                0.010364829627636777,
                -0.022437975652883912,
                0.0013571057232135628,
                0.015116558624286645,
                -0.0057093778245332575,
                -0.005605140249119078,
                -0.008611786322395995,
                0.009662062705362694,
                0.016109361624092232,
                -0.026551542376946853,
                -0.012987557415279204,
                -0.01146411314267011,
                0.01723487483428201,
                0.0027117296369333563,
                0.017728337426081108,
            ],
            index=range(24, 52),
            name="SMIO",
        )

        result = SMIO(
            self.tema_source,
            long_length=self.long_length,
            short_length=self.short_length,
            signal_length=self.signal_length,
            ma_method="tema",
        )

        pd.testing.assert_series_equal(result, expected_result)

    def test_smio_rma(self):
        expected_result = pd.Series(
            [
                np.nan,
                np.nan,
                np.nan,
                np.nan,
                -0.03702186972261484,
                -0.008146213601711881,
                -0.0233663444734149,
                -0.04874635380603385,
                -0.007990023275025897,
                -0.012629140500918175,
                -0.059853826546867245,
                -0.040324698413669216,
                -0.03029866612143188,
            ],
            index=range(24, 37),
            name="SMIO",
        )

        result = SMIO(
            self.source,
            long_length=self.long_length,
            short_length=self.short_length,
            signal_length=self.signal_length,
            ma_method="rma",
        )

        pd.testing.assert_series_equal(result, expected_result)

    def test_smio_invalid_method(self):
        with self.assertRaises(InvalidArgumentError) as context:
            SMIO(
                self.source,
                long_length=self.long_length,
                short_length=self.short_length,
                signal_length=self.signal_length,
                ma_method="invalid",
            )
        self.assertEqual(
            str(context.exception),
            "ma_method must be 'sma', 'ema', 'dema', 'tema', or 'rma', got 'invalid'.",
        )



import unittest

import pandas as pd
import numpy as np
from src.tradingview_indicators.moving_average import sma, ema, sema, rma
from src.tradingview_indicators.errors_exceptions import InvalidArgumentError


class TestMovingAverage(unittest.TestCase):
    def setUp(self):
        np.random.seed(42)
        self.source = pd.Series(np.random.randint(1, 500, 10))
        self.length = 5

    def test_sma(self):
        expected_result = pd.Series([253.2, 247.0, 197.6, 132.0, 98.4, 101.4])

        result = sma(self.source, self.length).reset_index(drop=True)

        pd.testing.assert_series_equal(result, expected_result)

    def test_ema(self):
        expected_result = pd.Series(
            [
                253.2,
                192.8,
                191.53333333333336,
                134.68888888888893,
                124.12592592592596,
                123.4172839506173,
            ]
        )

        result = ema(self.source, self.length).reset_index(drop=True)

        pd.testing.assert_series_equal(result, expected_result)

    def test_sema(self):
        smooth = 2

        expected_result = (
            pd.Series([179.26962962962963, 160.65218106995889])
            .rename("sema")
        )

        result = sema(self.source, self.length, smooth).reset_index(drop=True)

        pd.testing.assert_series_equal(result, expected_result)

    def test_rma_numpy(self):
        expected_result = pd.Series(
            [
                253.2,
                216.96,
                211.36800000000002,
                173.29440000000002,
                159.23552,
                151.788416,
            ]
        ).rename("RMA")

        result = rma(self.source, self.length, method="numpy").reset_index(drop=True)

        pd.testing.assert_series_equal(result, expected_result)

    def test_rma_pandas(self):
        expected_result = pd.Series(
            [
                np.nan,
                np.nan,
                np.nan,
                np.nan,
                253.2,
                152.53333333333333,
                167.47868852459015,
                117.85853658536587,
                113.43845787720134,
                115.75911180501345,
            ]
        ).rename("RMA")

        result = rma(self.source, self.length, method="pandas")

        pd.testing.assert_series_equal(result, expected_result)

    def test_rma_invalid_method(self):
        with self.assertRaises(InvalidArgumentError) as context:
            rma(self.source, self.length, method="invalid")
        self.assertEqual(
            str(context.exception),
            "method must be 'numpy' or 'pandas', got 'invalid'.",
        )



import unittest
import pandas as pd
import numpy as np
from src.tradingview_indicators.stoch import stoch


class TestStoch(unittest.TestCase):
    def setUp(self):
        np.random.seed(42)
        source = pd.read_csv("example/BTCUSDT_1d_spot.csv", index_col=0).iloc[:-1]
        self.short_source = source.iloc[:23]
        self.medium_source = source.iloc[:83]
        self.long_source = source.iloc[:119]

        self.k_length = 14
        self.k_smoothing = 1
        self.d_smoothing = 3

    def test_stoch(self):
        ref_indexes = pd.Index(
            [
                "2017-08-30",
                "2017-08-31",
                "2017-09-01",
                "2017-09-02",
                "2017-09-03",
                "2017-09-04",
                "2017-09-05",
                "2017-09-06",
                "2017-09-07",
                "2017-09-08",
            ],
            name="open_time",
        )

        ref_values = pd.Series(
            [
                92.59565053586745,
                98.47408244265733,
                96.59116152266836,
                69.65611782820838,
                72.05608144543561,
                45.48561256245167,
                57.13783219452327,
                76.09471706868041,
                81.47119795837419,
                50.87599817391242,
            ],
            index=ref_indexes,
            name="stoch",
        )

        test_stoch = stoch(
            self.short_source["close"],
            self.short_source["high"],
            self.short_source["low"],
            self.k_length,
        ).dropna()

        pd.testing.assert_series_equal(test_stoch, ref_values)



import unittest
import pandas as pd
import numpy as np
from src.tradingview_indicators.TRIX import TRIX
from src.tradingview_indicators.errors_exceptions import InvalidArgumentError


class TestTrix(unittest.TestCase):
    def setUp(self):
        np.random.seed(42)
        self.source = pd.Series(np.random.randint(1, 500, 60))
        self.dema_source = pd.Series(np.random.randint(1, 500, 112))
        self.tema_source = pd.Series(np.random.randint(1, 500, 163))
        self.length = 18
        self.signal_length = 1

    def test_trix_sma(self):
        expected_result = pd.Series(
            [
                np.nan,
                12.192185870230077,
                -1.6630617671520298,
                -15.837688425390795,
                -37.05532983268256,
                -51.953489146878695,
                -68.84957653841006,
                -77.75747400020627,
                -85.15236395226111,
            ],
            index=range(51, 60),
            name="TRIX",
        )

        result = TRIX(self.source, self.length, self.signal_length, "sma")

        pd.testing.assert_series_equal(result, expected_result)

    def test_trix_ema(self):
        expected_result = pd.Series(
            [
                np.nan,
                8.86050763307722,
                -1.9272532544878374,
                -4.833850140055773,
                -12.416765107685634,
                -7.833433279014201,
                -0.0018611586405370417,
                0.3318962104348344,
                -26.418522373559128,
            ],
            index=range(51, 60),
            name="TRIX",
        )

        result = TRIX(self.source, self.length, self.signal_length, "ema")

        pd.testing.assert_series_equal(result, expected_result)

    def test_trix_dema(self):
        expected_result = pd.Series(
            [
                np.nan,
                -37.34538675591814,
                -37.059518788655765,
                -36.94188499935969,
                -36.96795603815417,
                -37.090665032852854,
                -37.27604420752151,
                -37.48562142520839,
                -37.67782830019328,
                -37.82084645590267,
            ],
            index=range(102, 112),
            name="TRIX",
        )

        result = TRIX(self.dema_source, self.length, self.signal_length, "dema")

        pd.testing.assert_series_equal(result, expected_result)

    def test_trix_tema(self):
        expected_result = pd.Series(
            [
                np.nan,
                542.0515481781241,
                655.0753354089434,
                768.3775937297987,
                601.8218990305346,
                301.260699599446,
                157.91091562776137,
                -21.10271369721417,
                -99.46466831257439,
                -10.863910790561704,
            ],
            index=range(153, 163),
            name="TRIX",
        )

        result = TRIX(self.tema_source, self.length, self.signal_length, "tema")

        pd.testing.assert_series_equal(result, expected_result)

    def test_trix_rma(self):
        expected_result = pd.Series(
            [
                np.nan,
                9.90286601443735,
                8.71416291960081,
                8.408137074118471,
                7.315564899252891,
                7.813306810335874,
                8.860861098760608,
                8.951996463011369,
                5.044486316796437,
            ],
            index=range(51, 60),
            name="TRIX",
        )

        result = TRIX(self.source, self.length, self.signal_length, "rma")

        pd.testing.assert_series_equal(result, expected_result)

    def test_trix_invalid_method(self):
        with self.assertRaises(InvalidArgumentError) as context:
            TRIX(self.source, self.length, self.signal_length, "invalid")

        self.assertEqual(
            str(context.exception),
            "ma_method must be 'sma', 'ema', 'dema', 'tema', or 'rma', got 'invalid'.",
        )


from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

# Read requirements
with open("requirements.txt", "r") as req_file:
    requirements = [line.strip() for line in req_file if line.strip()]

VERSION = '0.1.2'
DESCRIPTION = 'An accurate calculation of technical analysis indicators with values aligning with those in TradingView.'

setup(
    name="tradingview_indicators",
    version=VERSION,
    author="Archie Marques",
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    python_requires=">=3.11",
    install_requires=requirements,
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3.11",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ],
    keywords=['python', 'tradingview', 'technical analysis', 'indicators']
)



import unittest
import pandas as pd
import numpy as np

from src.tradingview_indicators.bollinger import bollinger_trends
from src.tradingview_indicators.errors_exceptions import InvalidArgumentError

class TestBollingerBands(unittest.TestCase):
    def setUp(self):
        np.random.seed(42)
        self.source = pd.Series(np.random.randint(1, 500, 50))
        self.short_length = 10
        self.stdev = 2

    def test_bollinger_trends_normal(self):
        ref_values = [
            -73.09644670050763,
            -75.62002807674314,
            -84.45907635754708,
            87.6712328767123,
            131.4508264920836,
            -64.63473633284956,
            5.320945945945931,
            -7.547169811320736,
            24.2933537051184,
            -1.7156094986853856,
        ]

        ref_df = pd.Series(ref_values, index=range(9, 19), name="Bollinger Trend")

        test_values = bollinger_trends(
            self.source,
            self.short_length,
            self.stdev,
            stdev_method="absolute",
            diff_method="normal",
        ).iloc[9:19]

        pd.testing.assert_series_equal(ref_df, test_values)

    def test_bollinger_trends_absolute(self):
        ref_values = [
            -17859.6,
            -21531.6,
            -19321.82359030106,
            -18813.6,
            -20585.793575937514,
            -20803.6,
            -23667.4,
            -22807.2,
            -26116.4,
            -26674.575530532995,
        ]

        ref_df = pd.Series(
            ref_values,
            index=range(9, 19),
            name="Bollinger Trend",
        )

        test_values = bollinger_trends(
            self.source,
            self.short_length,
            self.stdev,
            stdev_method="absolute",
            diff_method="absolute",
        ).iloc[9:19]

        pd.testing.assert_series_equal(ref_df, test_values)

    def test_bollinger_trends_ratio(self):
        ref_values = [
            0.18461994200487786,
            0.3663345672678181,
            4.249053707103651,
            0.37280623343162145,
            0.2683303864537662,
            0.33307423920239476,
            0.23486902138749785,
            0.33951568314334674,
            0.12994393707976107,
            0.28554727862120927,
        ]

        ref_df = pd.Series(
            ref_values,
            index=range(9, 19),
            name="Bollinger Trend",
        )

        test_values = bollinger_trends(
            self.source,
            self.short_length,
            self.stdev,
            stdev_method="ratio",
            diff_method="ratio",
        ).iloc[9:19]

        pd.testing.assert_series_equal(ref_df, test_values)

    def test_bollinger_trends_dtw(self):
        ref_values = [
            18.461994200487784,
            9.390672023352204,
            96.56185881046612,
            93.60233380902268,
            104.61804735521858,
            109.2132021342806,
            90.07465530800913,
            74.30953900806698,
            60.265145210540304,
            43.246995578495266,
        ]

        ref_df = pd.Series(
            ref_values,
            index=range(9, 19),
            name="Bollinger Trend",
        )

        test_values = bollinger_trends(
            self.source,
            self.short_length,
            self.stdev,
            stdev_method="dtw",
            diff_method="dtw",
        ).iloc[9:19]

        pd.testing.assert_series_equal(ref_df, test_values)

    def test_bollinger_bands_invalid_ma_method(self):
        with self.assertRaises(InvalidArgumentError) as context:
            bollinger_trends(
                self.source,
                self.short_length,
                self.stdev,
                ma_method="invalid_method",
            )

        self.assertEqual(
            str(context.exception),
            "ma_method must be 'sma', 'ema', 'dema', 'tema', or 'rma', got 'invalid_method'.",
        )